// 'Meal' type has a method named '@@iterator' ('Symbol.iterator' <=> '@@iterator'), which is a generator function:
type Meal = {
    when: string, taste: string,
    [Symbol.iterator](): IterableIterator<string>
};

const meal: Meal = {
    when: "today", taste: "medium",
    [Symbol.iterator]: function* (from = "yesterday") { // Key 'Symbol.iterator' (i.e., '@@iterator') is added as 'Generator' object
        yield from;
        yield "today";
        yield "tomorrow";
    }
};
const {when: first_part, ...remaining_part} = meal;
console.info(first_part); // 'today'
console.info(JSON.stringify(remaining_part)); // '{"taste":"medium"}'

console.info(Object.getOwnPropertySymbols(meal)[0] === Symbol.iterator); // 'true'

const iterator: IterableIterator<string> = meal[Symbol.iterator]();
console.info(iterator.next().value); // 'yesterday'

// Spread syntax only applies on iterable objects (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Spread_syntax):
console.info([...meal]); // '[ 'yesterday', 'today', 'tomorrow' ]'

for (const value of meal)
    console.info(value); // 'yesterday' 'today' 'tomorrow'
